var storage = {
  get: function(key) {
    try {
      return JSON.parse(localStorage[key] || '{}');
    } catch(e) {
      return {};
    }
  },
  set: function(key, subkey, val) {
    var data = this.get(key);
    data[subkey] = val;
    localStorage[key] = JSON.stringify(data);
  },
};


function htmlEncode(str) {
  var div = document.createElement("div");
  var text = document.createTextNode(str);
  div.appendChild(text);
  var ret = div.innerHTML;
  ret = ret.replace(/"/g, '&quot;');
  return ret;
}

function checkUrlProtocol(url) {
  if (url) {
    if (url.substr(0, 7) == 'http://' || url.substr(0, 8) == 'https://' || url.substr(0, 9) == 'widget://') {
      return url;
    }
  }
  return '';
}

function filterUrl(url){
  return url.replace(filterUrl.filterRegex, '');
}

function filterParam(url) {
  return (url||'').replace(/\?.*$/ig, '');
}

$(function(){
  var st = Date.now();

  var lastApis = new ChromeWebUIApis({
    methods: 'queryUnClosedPages,deleteUnClosedPageById,clearAllUnClosedPages,getMostVisitedData,openUrl',
    onerror: function(ev) {
      console.log('error:', arguments);
    },
    onbefore: function(ev) {
      console.log('before call:' + ev.methodName, ev.args);
    }
  });
  window.lastApis = lastApis;


  var navItemFn = doT.template($('#nav-item').html()),
  urlListFn = doT.template($('#url-list').html());

  var dataDict = {};
  var reloadData = function(selectedDay){
    lastApis.queryUnClosedPages(function(data){
      console.log('queryUnClosedPages 回调函数被调用:', Date.now() - st + 'ms(距页面打开)', arguments);
      if (data.length == 0) {
        data = [{day:Date.now() / 1000, data:[]}];
        $('#clear-all,#btn-open-all').attr('disabled', 'disabled');
      } else {
        data = data.slice(0, 1);
      }

      $('ul.nav').hide();
/*
      if (data.length == 1) {
        $('ul.nav').hide();
      } else {
        $('ul.nav').show();
      }
*/
      $('.nav').html(navItemFn(data));

      dataDict = {};
      data.forEach(function(item){
        dataDict[item.day] = item.data;
      });

      var sel = $('.nav a[day=' + selectedDay + ']');
      if(sel.length == 0) {
        sel = $('.nav a:first');
      }
      sel.trigger('click');
    });

    lastApis.getMostVisitedData(function(tiles) {
      console.log('getMostVisitedData 回调函数被调用');
      console.log(tiles);
//      window.smartMostVisited = tiles;

      $('.smart-push').removeClass('active');
      var oftenLis = '', validCount = 0;
      //if (tiles.length > 8) tiles = tiles.slice(0, 8);
      for (var i = 0; i < tiles.length && validCount < 8; i++) {
        var tile = tiles[i];
        if (tile.title) {
          var mv_li = $('<li><img src="chrome://favicon/size/16/' + htmlEncode(checkUrlProtocol(tile.url)) +
                        '"><a href="' + htmlEncode(checkUrlProtocol(tile.url)) + '" target="_blank">' +
                        htmlEncode(tile.title) + '</a></li>');
          oftenLis += mv_li[0].outerHTML;
          validCount++;
        }
      }
      $('#mostvisited-box ul').html(oftenLis);
/*
      if (!smartPushInited && window.globalGridData && window.smartMostVisited) {
        smartPushInited = true;
        buildSmartPush();
      }
*/
    });
    return arguments.callee;
  }();

  function saveSetting(item, exec) {
    var settings = localStorage['settings'] = localStorage['settings'] || '{}';
    try {
      settings = JSON.parse(settings);
    } catch(e) {
      settings = {};
    }

    settings[item] = exec;

    localStorage['settings'] = JSON.stringify(settings);
  }

  function loadSettings() {
    var settings = localStorage['settings'] = localStorage['settings'] || '{}';
    try {
      settings = JSON.parse(settings);
    } catch(e) {
      settings = {};
    }
    for (var k in settings) {
      eval(settings[k]);
    }
  }

  function setSearch(searchItem) {
    $('#search-switch>a')[0].className = searchItem.type;
    $('#search-form').attr('action', searchItem.url);
    $('#search-kw').attr('name', searchItem.key);
    $('#search-kw').attr('placeholder', searchItem.desc);

    $('#search-form input[type=hidden]').remove();

    var params;
    if (params = searchItem.params) {
      for (var k in params) {
        var hInput = document.createElement('input');
        hInput.type = 'hidden';
        hInput.name = k;
        hInput.value = params[k];
        $('#search-form').append(hInput);
      }
    }

    /*if ($('#search-kw').hasClass('input-txt-place')) {
      PlaceHolder.create(document.getElementById('search-kw'));
    }*/
  }

  $('#search-switch').live('click', function(e) {
    var self = $(this),
    offset = self.offset();
    var menu = $('.search-menu').css({
      left: offset.left,
      top: offset.top + self.height()
    });
    if ($('.ac_results').is(':visible')) {
      $('.ac_results').hide();
      menu.slideDown(400, function(){
        $(this).addClass('open');
      });
    } else if (menu.hasClass('open')) {
      menu.slideUp(400, function(){
        $(this).removeClass('open');
      });
    } else {
      menu.slideDown(400, function(){
        $(this).addClass('open');
      });
    }
  });

  function renderDay(day) {
    var INTERVAL = 60;
    var list = dataDict[day];

    var group = [];
    var firstTime = Number.MAX_VALUE;
    var map = {};
    list.forEach(function(item){
      if ((firstTime - item.time) > INTERVAL) {
        firstTime = item.time;
        group.push([]);
        map = {};
      }
      if (map[item.url]) {
        lastApis.deleteUnClosedPageById(item.id);
      } else {
        map[item.url] = true;
        group[group.length-1].push(item);
      }
    });

    $('#list-last').html(urlListFn([group[0]]));
    $('#list-more').html(urlListFn(group.slice(1, group.length)));
/*
    if (group.length > 1) {
      $('div.more').show();
    } else {
      $('div.more').hide();
    }
*/
    if (!group[0] || group[0].length == 0) {
      $('#empty-data').show();
    }
  }

  $('#clear-all').click(function(){
    lastApis.clearAllUnClosedPages(function(){
      reloadData();
    });
  });
  $('#goto-history').click(function(){
    lastApis.openUrl('se://history/', 0, 0, 0);
  });
  $('#btn-open-all').click(function(){
    $('#list-last li div a').each(function(){
      $(this).css('color', '#999');
      lastApis.openUrl($(this).attr('href'), 2, 2, 0);
    });
  });
  $('.list li a[href]').live('click', function(){
    $(this).css('color', '#999');
    var url = $(this).attr('href') || '';
    if (url.substr(0, 7) == 'file://') {
      lastApis.openUrl(url, 1, 2, 0);
      return false;
    }
  });
  $('#btn-more').click(function(){
    $('#list-more').slideToggle('fast');
  });
  $('.nav a').live('click', function(){
    $('.nav a').removeClass('selected');
    $(this).addClass('selected');
    renderDay($(this).attr('day'));
  });
  $('.remove').live('click', function(){
    lastApis.deleteUnClosedPageById(parseInt($(this).attr('url-id')), function(){
      reloadData($('.nav a.selected').attr('day'));
    });
    $(this).parents('li').remove();
  });


  var searchMaps = {
    so: {
      type: 'so',
      desc: '360搜索',
      url: 'http://www.so.com/s?src=se6_newtab&ie=utf-8',
      key: 'q',
      params: {
        src: 'se6_newtab',
        ie: 'utf-8'
      }
    },
    baidu: {
      type: 'baidu',
      desc: '百度',
      url: 'http://www.baidu.com/baidu?&ie=utf-8',
      key: 'word',
      params: {
        ie: 'utf-8'
      }
    },
    google: {
      type: 'google',
      desc: '谷歌',
      url: 'http://www.google.com.hk/search?client=aff-cs-360se&ie=UTF-8',
      key: 'q',
      params: {
        client: 'aff-cs-360se',
        ie: 'UTF-8'
      }
    },

    news_so: {
      type: 'news_so',
      desc: '360搜索',
      url: 'http://news.so.360.cn/ns?ie=utf-8&tn=news&src=se6_newtab',
      key: 'q',
      params: {
        src: 'se6_newtab',
        ie: 'utf-8'
      }
    },
    news_baidu: {
      type: 'news_baidu',
      desc: '百度',
      url: 'http://news.baidu.com/ns?&ie=utf-8',
      key: 'word',
      params: {
        ie: 'utf-8'
      }
    },
    news_google: {
      type: 'news_google',
      desc: '谷歌',
      url: 'http://news.google.com.hk/news/search?client=aff-cs-360se&ie=UTF-8',
      key: 'q',
      params: {
        client: 'aff-cs-360se',
        ie: 'UTF-8'
      }
    },

    video_so: {
      type: 'video_so',
      desc: '360搜索',
      url: 'http://video.so.com/mini.php?src=se6_newtab&ie=utf-8',
      key: 'kw',
      params: {
        src: 'se6_newtab',
        ie: 'utf-8'
      }
    },
    video_baidu: {
      type: 'video_baidu',
      desc: '百度',
      url: 'http://video.baidu.com/v?&ie=utf-8',
      key: 'word',
      params: {
        ie: 'utf-8'
      }
    },
    video_google: {
      type: 'video_google',
      desc: '谷歌',
      url: 'http://www.google.com.hk/search?client=aff-cs-360se&ie=UTF-8',
      key: 'q',
      params: {
        tbm: 'vid',
        client: 'aff-cs-360se',
        ie: 'UTF-8'
      }
    },

    image_so: {
      type: 'image_so',
      desc: '360搜索',
      url: 'http://image.so.com/i?src=se6_newtab&ie=utf-8',
      key: 'q',
      params: {
        src: 'se6_newtab',
        ie: 'utf-8'
      }
    },
    image_baidu: {
      type: 'image_baidu',
      desc: '百度',
      url: 'http://image.baidu.com/i?&ie=utf-8',
      key: 'word',
      params: {
        ie: 'utf-8'
      }
    },
    image_google: {
      type: 'image_google',
      desc: '谷歌',
      url: 'http://images.google.com.hk/images?client=aff-cs-360se&ie=UTF-8',
      key: 'q',
      params: {
        client: 'aff-cs-360se',
        ie: 'UTF-8'
      }
    },

    music_so: {
      type: 'music_so',
      desc: '360搜索',
      url: 'http://s.music.so.com/s?src=se6_newtab&ie=utf-8',
      key: 'q',
      params: {
        src: 'se6_newtab',
        ie: 'utf-8'
      }
    },
    music_baidu: {
      type: 'music_baidu',
      desc: '百度',
      url: 'http://music.baidu.com/search?&ie=utf-8',
      key: 'key',
      params: {
        ie: 'utf-8'
      }
    },
    music_sogou: {
      type: 'music_sogou',
      desc: '搜狗',
      url: 'http://mp3.sogou.com/music?ie=UTF-8',
      key: 'query',
      params: {
        ie: 'UTF-8'
      }
    },

    map_so: {
      type: 'map_so',
      desc: '360搜索',
      url: 'http://map.so.com/?src=se6_newtab&ie=utf-8&t=map',
      key: 'k',
      params: {
        src: 'se6_newtab',
        ie: 'utf-8',
        t: 'map'
      }
    },
    map_baidu: {
      type: 'map_baidu',
      desc: '百度',
      url: 'http://map.baidu.com/m?&ie=utf-8',
      key: 'word',
      params: {
        ie: 'utf-8'
      }
    },
    map_google: {
      type: 'map_google',
      desc: '谷歌',
      url: 'http://ditu.google.cn/maps?client=aff-cs-360se&ie=UTF-8',
      key: 'q',
      params: {
        client: 'aff-cs-360se',
        ie: 'UTF-8'
      }
    },

    wenda_so: {
      type: 'wenda_so',
      desc: '360问答',
      url: 'http://wenda.qihoo.com/search/?src=se6_newtab&ie=utf-8',
      key: 'q',
      params: {
        src: 'se6_newtab',
        ie: 'utf-8'
      }
    },
    wenda_baidu: {
      type: 'wenda_baidu',
      desc: '百度',
      url: 'http://zhidao.baidu.com/search?&ie=utf-8',
      key: 'word',
      params: {
        ie: 'utf-8'
      }
    }
  };

  var searchTypeMaps = {
    webpage: ['so', 'google', 'baidu'],
    news: ['news_so', 'news_google', 'news_baidu'],
    video: ['video_so', 'video_google', 'video_baidu'],
    image: ['image_so', 'image_google', 'image_baidu'],
    music: ['music_so', 'music_sogou', 'music_baidu'],
    map: ['map_so', 'map_google', 'map_baidu'],
    wenda: ['wenda_so', 'wenda_baidu']
  };

  function setSearchType(type) {
    $('.search-cat > a').removeClass('on').filter('[cat-name=' + type + ']').addClass('on');

    var engines;
    if (engines = searchTypeMaps[type]) {
      $('.search-menu').empty();
      engines.forEach(function(sn) {
        var item = searchMaps[sn] || {};
        $('.search-menu').append('<li><a __href="#" class="' + item.type + '"></a>' + item.desc + '</li>');
      });
      var jitem = $('.search-menu .' + storage.get('default_engine')[type]);
      if (jitem.length == 0) {
        jitem = $('.search-menu a:first');
      }
      console.log(jitem);
      jitem.trigger('click');

      saveSetting('search_type', 'setSearchType("' + type + '")');

      return true;
    }
  }

  $('.search-cat > a').live('click', function() {
    if (setSearchType($(this).attr('cat-name'))) {

    }
  });

  $('.search-menu li').live('click', function(e) {
    var searchItem;
    if (searchItem = searchMaps[e.target.className] || searchMaps[e.target.firstElementChild.className]) {

      setSearch(searchItem);
      $('.search-menu').slideUp(200, function(){
        $(this).removeClass('open');
      });

      saveSetting('search', 'setSearch(' + JSON.stringify(searchItem) + ');');
      storage.set('default_engine', $('.search-cat .on').attr('cat-name'), searchItem.type);
    }
  });

  //PlaceHolder.create($('#search-kw, .placeholder'));

  $('#search-form').live('submit', function() {
    var input = $('#search-kw');
    if (input.hasClass('input-txt-place')) {
      input.val('');
    }
    input.val(filterUrl(input.val()));

    Stat.count('d2', $('.search-cat .on').index() * 4 + 11);
    Stat.searchTypeCount();
  });

  $(document).on('click', function(e) {
    if (e.target.id != 'search-switch' && ! $(e.target).parents('#search-switch').length && ! $(e.target).parents('.search-menu').length) {
      setTimeout(function(){
        var searchMenu = $('.search-menu');
        if(!searchMenu.hasClass('is-sliding')){
          searchMenu.slideUp(100, function(){
            $(this).removeClass('open');
          });
        }
      }, 10);
    }
  });

  $('#search-kw').autocomplete('http://sug.so.360.cn/suggest?encodein=utf-8&encodeout=utf-8', {
    selectFirst: false,
    dataType: 'jsonp',
    scrollHeight: 300,
    onbeforeshow: function(callback){
      var searchMenu = $('.search-menu');
      searchMenu.hide();
      callback && callback();
      return true;
      if(searchMenu.is(":visible")){
        searchMenu.addClass('is-sliding');
        searchMenu.slideUp(200, function(){
          $(this).removeClass('open');
          setTimeout(function(){
            searchMenu.removeClass('is-sliding');
            callback && callback();
          }, 0)
        })
      }else{
        callback && callback();
      }
    },
    parse: function(data) {
      var parsed = [];
      data.s.forEach(function(v) {
        parsed.push({
          data: [v],
          value: v,
          result: v
        });
      });
      return parsed;
    }
  }).on('result', function(e, data) {
    if (data.text) {
      Stat.count('d2', $('.search-cat .on').index() * 4 + 14);
    } else {
      Stat.count('d2', $('.search-cat .on').index() * 4 + 12);
    }
    if (data.link) {
      location = data.link;
      return;
    }
    Stat.searchTypeCount();
    $(this).parents('form')[0].submit();
  });

  loadSettings();

});

function formatNavDay(timestamp) {
  timestamp = timestamp * 1000;
  var now = new Date();
  var d = new Date(timestamp);
  var nowDay = Math.floor((now.getTime() + 1000 *60*60*8) / (1000 *60*60*24));
  var dDay = Math.floor((d.getTime() + 1000 *60*60*8) / (1000 *60*60*24));
  var arr = ['今天', '昨天', '前天'];
  var ret = arr[nowDay - dDay];
  if (!ret) {
    ret = (d.getMonth()+1) + '月' + d.getDate() + '日';
  }
  return ret;
}

function formatTime(timestamp) {
  timestamp = timestamp * 1000;
  var d = new Date(timestamp);
  var m = d.getMinutes();
  if (m < 10) {
    m = '0' + m;
  }
  return d.getHours() + ':' + m;
}
